'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tape = _interopDefault(require('tape'));
var jStat = _interopDefault(require('jstat'));
var stdlib = _interopDefault(require('@stdlib/stdlib'));

function max(x) {
  return jStat.max(x)
}

function mean(x) {
  return jStat.mean(x)
}

function median(x) {
  return jStat.median(x)
}

function min(x) {
  return jStat.min(x)
}

function mode(x) {
  return jStat.mode(x)
}

function sum(x) {
  return jStat.sum(x)
}

function ttest(x, y, mu, tails, paired, alpha) {
  const implem = stdlib.math.stats.ttest;
  const options = {
    mu: mu || 0,
    alternative: ['less', 'two-sided', 'greater'][(tails || 0) + 1],
    alpha: alpha || 0.05,
  };
  let result;
  if (paired) result = implem(x, y, options);
  else {
    if (y) result = implem(x, Object.assign(options, { mu: mean(y) + options.mu }));
    else result = implem(x, options);
  }
  return {
    _class: 'ttest',
    mean: result.mean,
    sd: result.sd,
    statistic: result.statistic,
    ci: result.ci,
    df: result.df,
    p_value: result.pValue,
    rejected: result.rejected
  }
}

// Don't change these unless you want to
// fix a lot of the tests below!
const x = [0,1,2,3,4,5,6,7,8,9];
const y = [9,8,7,6,5,4,3,2,1,0];
const a = [0,1,2,4,4,4,4,2,1,0];

tape('max', function (assert) {
    assert.equal(max(x), 9);
    assert.end();
});

tape('mean', function (assert) {
    assert.equal(mean(x), 4.5);
    assert.end();
});

tape('median', function (assert) {
    assert.equal(median(x), 4.5);
    assert.end();
});

tape('min', function (assert) {
    assert.equal(min(x), 0);
    assert.end();
});

tape('mode', function (assert) {
    assert.equal(mode(a), 4);
    assert.end();
});

tape('sum', function (assert) {
    assert.equal(sum(x), 45);
    assert.end();
});

tape('ttest', function (assert) {
	const unpaired = ttest(x, y);
    assert.equal(unpaired._class, "ttest");
    assert.equal(unpaired.statistic, 0);

    const paired = ttest(x, y, 0, 0, true);
    assert.equal(paired.df, 9);

    assert.end();
});

function type (value) {
  let type = typeof value;

  if (value === null) {
    return 'null'
  } else if (type === 'boolean') {
    return 'boolean'
  } else if (type === 'number') {
    let isInteger = false;
    if (value.isInteger) isInteger = value.isInteger();
    else isInteger = (value % 1) === 0;
    return isInteger ? 'integer' : 'number'
  } else if (type === 'string') {
    return 'string'
  } else if (type === 'object') {
    if (value.constructor === Array) return 'array'
    else return 'object'
  } else {
    return 'unknown'
  }
}

tape('type', function (assert) {
  assert.equal(type(null), 'null');
  assert.equal(type(true), 'boolean');
  assert.equal(type(3.14), 'number');
  assert.equal(type(42), 'integer');
  assert.equal(type('hello'), 'string');

  assert.equal(type([]), 'array');

  assert.equal(type({}), 'object');

  assert.end();
});
